if ( ! Detector.webgl ) Detector.addGetWebGLMessage({id:"placeholder"});

var dim = 1200;
var dimMax = 1200;
var scene = new THREE.Scene();
var camera = new THREE.OrthographicCamera(-1, 1, -1, 1, -1, 1);
var renderer = new THREE.WebGLRenderer();
var uniforms;

var mouseDown = false;
var lastMouseX = null;
var lastMouseY = null;

var curLatitude = 0.0;
var curLongitude = 0.0;

// http://www.khronos.org/message_boards/showthread.php/7170-How-to-include-shaders
function getSourceSync(url) {
  var req = new XMLHttpRequest();
  req.open("GET", url, false);
  req.send(null);
  return (req.status == 200) ? req.responseText : null;
};

function deg2rad(deg) {
    return deg*(Math.PI/180);
}

function rad2deg(rad) {
    return rad*(180/Math.PI);
}

function dragDown(x,y){
    mouseDown = true;
    lastMouseX = x;
    lastMouseY = y;

}

function dragUp(x,y){
    mouseDown = false;
}

function dragMove(x,y){

    if  (!mouseDown) {
        return;
    }

    var newX = x;
    var newY = y;

    var deltaX = newX - lastMouseX
    var deltaY = newY - lastMouseY;

    //var cur_latitude = deg2rad(parseFloat(document.getElementById("cur_latitude").value));
    //var cur_longitude = deg2rad(parseFloat(document.getElementById("cur_longitude").value));

    curLatitude += deltaY/100.0;
    curLongitude -= deltaX/100.0;

    //document.getElementById("cur_latitude").value = rad2deg(cur_latitude);
    //document.getElementById("cur_longitude").value = rad2deg(cur_longitude);

    lastMouseX = newX
    lastMouseY = newY;

}

function handleMouseDown(event) {
    dragDown(event.clientX,event.clientY);
}

function handleMouseUp(event) {
    dragUp(event.clientX,event.clientY);
}

function handleMouseMove(event) {
    dragMove(event.clientX,event.clientY);
}


function onDocumentTouchStart( event ) {

    if ( event.touches.length === 1 ) {
        event.preventDefault();

        dragDown(event.touches[ 0 ].pageX, event.touches[0].pageY)
    }
}

function onDocumentTouchMove( event ) {
    if ( event.touches.length === 1 ) {

        event.preventDefault();

        dragMove(event.touches[ 0 ].pageX, event.touches[0].pageY)

    }

}

function onDocumentTouchEnd( event ) {
/*
    if ( event.touches.length === 1 ) {

        event.preventDefault();

        dragUp(event.touches[ 0 ].pageX, event.touches[0].pageY)
    }
    */

}

function resizeCanvas(w,h)
{
    renderer.setSize(w,h);

}


function initCanvas() {

    window.addEventListener('resize', function () {
        debugger;

   //     console.log("Resize: "+window.innerWidth+","+window.innerHeight);

        dim = Math.min(window.innerWidth, window.innerHeight, dimMax);

        document.getElementById("windowSize").innerHTML = ''+window.innerWidth+','+ window.innerHeight;

        resizeCanvas(dim,dim);
//        camera.aspect = window.innerWidth / window.innerHeight;
//        camera.updateProjectionMatrix();
    });

    window.addEventListener('orientationchange', function () {
        debugger;

        console.log("Orient: "+window.innerWidth+","+window.innerHeight);

        dim = Math.min(window.innerWidth, window.innerHeight, dimMax);

        document.getElementById("windowSize").innerHTML = ''+window.innerWidth+','+ window.innerHeight;

        resizeCanvas(dim,dim);
//        camera.aspect = window.innerWidth / window.innerHeight;
//        camera.updateProjectionMatrix();
    });

    debugger;
    dim = Math.min(window.innerWidth, window.innerHeight, dimMax);
    resizeCanvas(dim,dim);

    div = document.getElementById("placeholder");
    div.appendChild(renderer.domElement);



    renderer.domElement.onmousedown = handleMouseDown;
    document.onmouseup = handleMouseUp;
    document.onmousemove = handleMouseMove;

    document.addEventListener( 'touchstart', onDocumentTouchStart, false );
    document.addEventListener( 'touchmove', onDocumentTouchMove, false );
    document.addEventListener( 'touchend', onDocumentTouchEnd, false );

    var worldTexture = THREE.ImageUtils.loadTexture('assets/image/world1024x512.jpg');
    // without this, there is a line where the world wraps in the Pacific
    worldTexture.minFilter = THREE.NearestFilter;
    // need wrapping enabled
    worldTexture.wrapS = THREE.RepeatWrapping;
    worldTexture.wrapT = THREE.RepeatWrapping;
    uniforms = {
        texture1: {type: "t", value: worldTexture},
        phi1:     {type: "f", value: 0.0},
        lambda0:  {type: "f", value: 0.0}
    };
    var vertShader = getSourceSync("assets/js/azeqproj_vs.glsl");
    var fragShader = getSourceSync("assets/js/azeqproj_fs.glsl");

    var mapMaterial = new THREE.ShaderMaterial({
        side:           THREE.DoubleSide,
        uniforms:       uniforms,
        vertexShader:   vertShader,
        fragmentShader: fragShader
    });

    var mapGeometry = new THREE.PlaneGeometry(2, 2, 1, 1);
    mapGeometry.faceVertexUvs[0] = [];
    mapGeometry.faceVertexUvs[0].push([
        new THREE.Vector2(0, 0),
        new THREE.Vector2(0, 1),
        new THREE.Vector2(1, 0)
    ]);
    mapGeometry.faceVertexUvs[0].push([
        new THREE.Vector2(0, 1),
        new THREE.Vector2(1, 1),
        new THREE.Vector2(1, 0)
    ]);
    var map = new THREE.Mesh(mapGeometry, mapMaterial);
    scene.add(map);


    var lineMaterial = new THREE.LineBasicMaterial({color: 0xffff00});
    var crossGeometry = new THREE.Geometry();
    var crossDelta = 0.02;
    crossGeometry.vertices.push(new THREE.Vector3(-crossDelta, 0, 0.1));
    crossGeometry.vertices.push(new THREE.Vector3(crossDelta, 0, 0.1));
    crossGeometry.vertices.push(new THREE.Vector3(0,-crossDelta, 0.1));
    crossGeometry.vertices.push(new THREE.Vector3(0, crossDelta, 0.1));
    var cross = new THREE.Line(crossGeometry, lineMaterial, THREE.LinePieces);
    scene.add(cross);

    // Equatorial Circumference of the earth = 40,075.017 km
    var COE = 40075.017;
    var circleStep = 5000;
    for(var km = circleStep; km < COE; km += circleStep) {
        var i = km/COE;
        var circleGeometry = new THREE.CircleGeometry(i, 100);
        circleGeometry.vertices.shift(); // pop off the circle center
        circleGeometry.vertices.verticesNeedUpdate;
        var circle = new THREE.Line(circleGeometry, lineMaterial);
        circle.translateZ(0.1);
        scene.add(circle);
    }


}

var render = function () {
    requestAnimationFrame(render);
    uniforms.phi1.value = curLatitude;
    uniforms.lambda0.value = curLongitude;
    renderer.render(scene, camera);
};

initCanvas();
render();
